from enum import Enum


class CodeEnum(Enum):
    """
    word操作类型
    """
    SUCCESS = "SUCCESS"
    REEOR = "REEOR"
    PARAM_ERROR = "PARAM_ERROR"
    INNER_REEOR = "INNER_REEOR"

class CodeEnum:
    __doc__ = '设置统一的code返回值'
    error = -1  # 查询错误
    success = 0  # 成功
    insufficient_permissions = 1  # 权限不足
    data_duplication = 2  # 数据重复
    repeat_operation = 3  # 重复操作
    data_not_exist = 4  # 数据不存在
    parameter_error = 5  # 参数错误
    space_error = 6  # 空间不足
    state_error = 7  # 状态异常
    connection_data = 8  # 数据存在关联
    task_run = 9  # 任务正在执行
# 定义结果状态枚举
RESULT_SUCCESS = "SUCCESS"
RESULT_ERROR = "ERROR"

class ResultStatus(Enum):
    SUCCESS = "SUCCESS"
    ERROR = "ERROR"

class BusinessCode(Enum):
    OK = "OK"
    INNER_ERROR = "InnerError"
    FORBIDDEN = "Forbidden"
    NOT_FOUND = "NotFound"
    BAD_REQUEST = "BadRequest"

from enum import IntEnum

class HttpStatusCode(IntEnum):
    # 信息性状态码
    CONTINUE = 100
    SWITCHING_PROTOCOLS = 101

    # 成功状态码
    OK = 200
    CREATED = 201
    NO_CONTENT = 204

    # 重定向状态码
    MOVED_PERMANENTLY = 301
    FOUND = 302
    NOT_MODIFIED = 304

    # 客户端错误状态码
    BAD_REQUEST = 400
    UNAUTHORIZED = 401
    FORBIDDEN = 403
    NOT_FOUND = 404
    METHOD_NOT_ALLOWED = 405

    # 服务器错误状态码
    INTERNAL_SERVER_ERROR = 500
    BAD_GATEWAY = 502
    SERVICE_UNAVAILABLE = 503
    GATEWAY_TIMEOUT = 504

    @classmethod
    def get_description(cls, status_code):
        descriptions = {
            cls.CONTINUE: "客户端可以继续发送请求的剩余部分，或者忽略这个响应直接发送请求。",
            cls.SWITCHING_PROTOCOLS: "服务器理解并同意客户端切换协议的请求，后续将使用新协议进行通信。",
            cls.OK: "请求成功，服务器已成功处理请求并返回结果。",
            cls.CREATED: "请求成功，并且服务器已经创建了新的资源。通常在 POST 请求创建资源时使用。",
            cls.NO_CONTENT: "请求成功，但响应中没有返回任何内容。",
            cls.MOVED_PERMANENTLY: "请求的资源已永久移动到新的 URL，后续请求应使用新的 URL。",
            cls.FOUND: "请求的资源临时移动到了新的 URL，客户端应继续使用原 URL 进行后续请求。",
            cls.NOT_MODIFIED: "客户端发送的请求带有缓存验证信息（如 `If-Modified-Since` 或 `If-None-Match`），服务器验证后发现资源未被修改，客户端可以使用本地缓存的版本。",
            cls.BAD_REQUEST: "客户端发送的请求语法错误，不能被服务器所识别。",
            cls.UNAUTHORIZED: "请求需要用户进行身份验证，但客户端未提供有效的身份验证信息。",
            cls.FORBIDDEN: "服务器理解请求客户端的请求，但是拒绝执行此请求。",
            cls.NOT_FOUND: "请求的资源不存在，服务器无法找到对应的资源。",
            cls.METHOD_NOT_ALLOWED: "客户端使用的请求方法（如 GET、POST 等）不被该资源支持。",
            cls.INTERNAL_SERVER_ERROR: "服务器内部发生错误，无法完成请求。",
            cls.BAD_GATEWAY: "服务器作为网关或代理，从上游服务器收到了无效的响应。",
            cls.SERVICE_UNAVAILABLE: "服务器目前无法处理请求，通常是由于服务器过载或正在进行维护。",
            cls.GATEWAY_TIMEOUT: "服务器作为网关或代理，未能及时从上游服务器收到响应。"
        }
        return descriptions.get(status_code, "未知状态码")

class BusinessStatusCode(IntEnum):
    SUCCESS = 2000
    """业务处理成功。"""
    DATA_QUERY_SUCCESS = 2001
    """数据查询成功。"""
    BUSINESS_PARAM_ERROR = 4000
    """业务参数错误，请检查输入。"""
    DATA_ALREADY_EXISTS = 4001
    """数据已存在，请勿重复创建。"""
    BUSINESS_PROCESS_EXCEPTION = 5000
    """业务处理异常，请稍后重试。"""

    @classmethod
    def get_description(cls, code):
        member = cls._member_map_.get(code)
        if member:
            doc = member.__doc__
            return doc if doc else "无描述信息"
        if code == 1:
            return cls.SUCCESS
        if code == 2:
            return ""
        return "未知状态码"


import traceback

from pydantic.main import BaseModel

from sthg_fastapi_logs.exception import register_log_exception

from sthg_fastapi_logs.log_wrapper import register_log_middleware, init_log_config, access_log, service_log
import uvicorn
from fastapi import FastAPI, HTTPException

app = FastAPI()
config = {
    "is_console": True,
}
init_log_config()
register_log_exception(app)
register_log_middleware(app)

from sthg_fastapi_logs.log_wrapper import class_log


@app.get("/error_ValueError")
@access_log(print_res=True, print_req=True)
async def server1():
    t = Test()
    t.get_user_name(a='222222222222')
    t.get_user_id('s', b='11111')
    raise ValueError("这是一个错误")


@app.get("/error_HTTPException")
@access_log(print_res=True)
async def server2():
    raise HTTPException(status_code=403, detail={'msg': "用户无权限"})


@app.get("/not_reponse")
@access_log(print_res=True)
async def server3():
    t = Test()

    t.get_user_name(a='222222222222')
    t.get_user_id(s="s", b='11111')
    return None


@app.get("/reponse_is_list")
@access_log(print_res=True)
async def server4():
    t = Test()
    t.get_user_name(a='222222222222')
    # t.get_user_id("s",b='11111')
    return {"code": 0, "data": {}, "message": "获取规则执行记录 成功", "total": 0}


class Question(BaseModel):
    question: str


@app.post("/reponse_is_dict")
@access_log(print_res=True)
async def server5(obj: Question):
    t = Test()
    t.get_user_name(a='222222222222')
    t.get_user_id("s", b='11111')

    return {
        "data": 200,
        "code": 0,
        "message": "11111"
    }


@app.post("/testcdm")
@access_log(print_res=True)
async def server6(obj: Question):
    t = Test()
    t.get_user_name(a='222222222222')
    t.get_user_id(b='11111')
    return {"Hello": "World", "id": 1}


@app.post("/testservice")
@access_log(print_res=True)
@service_log()
async def server7(obj: Question):

    return {"code": 0, "data": {}, "message": "成功", "total": 0}


@app.post("/testservice1")
@access_log()
@service_log()
async def server8(obj: Question):
    t = Test()
    t.get_user_name(a='222222222222')
    t.get_user_id("s", b='11111')
    return {"code": 0, "data": {}, "message": "成功", "total": 0}


@class_log
class Test():
    def __init__(self):
        self.a = 123
        pass

    def get_user_name(self, a):
        return a

    def get_user_id(self, s, b):
        return b



class NewTest():
    def __init__(self):
        self.a = 456

    def get_user_a_NewTest(self, a):
        return a

    @service_log()
    def get_user_id_NewTest(self, s, b):
        raise ValueError('出错了aaaa')


if __name__ == '__main__':
    uvicorn.run('app_view:app', port=8030, host='0.0.0.0', proxy_headers=False, debug=True,
                timeout_keep_alive=300)



"""
@Author  ：duomei
@File    ：test_app.py
@Time    ：2025/3/14 9:39
"""

import os
import pytest
from starlette.testclient import TestClient

from app.app_server.app_view import app
from sthg_fastapi_logs.utils.exception_utils import CustomException

headers = {
    "Authorization": os.getenv("Authorization",
                               "bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJsb2dpblR5cGUiOiJsb2dpbiIsImxvZ2luSWQiOjI1MCwicm5TdHIiOiJtV24yekhEemk3bER6SlNKVVZ6RGVDeFlhRTc1YWUyTiIsIm5hbWUiOiJsanEiLCJwYXN0VGltZSI6IjIwMjQtMTEtMjYgMTY6MjA6NTQifQ.3Tv3AP0Mkaa58kl7xE1-e-Fnbnt4RpBYLTYTT5Dv8Xk"),
    "Satoken": os.getenv("Satoken",
                         "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJsb2dpblR5cGUiOiJsb2dpbiIsImxvZ2luSWQiOjI1MCwicm5TdHIiOiJtV24yekhEemk3bER6SlNKVVZ6RGVDeFlhRTc1YWUyTiIsIm5hbWUiOiJsanEiLCJwYXN0VGltZSI6IjIwMjQtMTEtMjYgMTY6MjA6NTQifQ.3Tv3AP0Mkaa58kl7xE1-e-Fnbnt4RpBYLTYTT5Dv8Xk"),
    "Custom-Header": os.getenv("Custom-Header", "custom_value")
}



@pytest.fixture
def client():
    """
    定义一个 pytest fixture 用于提供 TestClient 实例
    """
    return TestClient(app)



# 接口内部异常
#
def test_error_ValueError(client):
    """
    测试获取全部数据源类型接口
    """
    params = {
        "nihao":"你好",
        "shijie":"世界"
    }
    try:
        # 发起 post 请求
        response = client.get(
            "/error_ValueError",
            headers=headers,
            params=params
        )

        # 验证响应状态码
        assert response.status_code == 405
    except  ValueError as e:
        assert '这是一个错误' in str(e)

def test_error_ValueError1(client):
    """
    测试获取全部数据源类型接口
    """
    # params = {
    #     "nihao":"你好",
    #     "shijie":"世界"
    # }
    try:
        # 发起 post 请求
        response = client.get(
            "/error_ValueError",
            headers=headers,
            # params=params
        )

        # 验证响应状态码
        assert response.status_code == 405
    except  ValueError as e:
        assert '这是一个错误' in str(e)


def test_reponse_is_dict(client):
    """
    测试获取全部数据源类型接口
    """
    params = {
        "nihao":"你好",
        "shijie":"世界"
    }
    try:
        # 发起 post 请求
        response = client.get(
            "/reponse_is_dict",
            headers=headers,
            params=params
        )

        # 验证响应状态码
        assert response.status_code == 405
    except Exception as e:
        print(e)


# http请求异常

def test_error_HTTPException(client):
    """
    测试获取全部数据源类型接口
    """
    # params = {
    #     "nihao":"你好",
    #     "shijie":"世界"
    # }
    try:
        # 发起 post 请求
        response = client.get(
            "/error_HTTPException",
            headers=headers,
            # params=params
        )

        # 验证响应状态码
        assert response.status_code == 500
    except  ValueError as e:
        assert '出错了aaaa' in str(e)
    except Exception as e:
        print(e)
#返回空  返回值格式非标准
def test_not_reponse(client):
    """
    测试获取全部数据源类型接口
    """
    # params = {
    #     "nihao":"你好",
    #     "shijie":"世界"
    # }
    try:
        # 发起 post 请求
        response = client.get(
            "/not_reponse",
            headers=headers,
            # params=params
        )

        # 验证响应状态码
        assert response.status_code == 403
    except CustomException as e:
        assert "返回结构不是标准结构" in str(e)
    except  ValueError as e:
        print(e)
# 返回正常
def test_reponse_is_list(client):
    """
    测试获取全部数据源类型接口
    """
    # params = {
    #     "nihao":"你好",
    #     "shijie":"世界"
    # }
    try:
        # 发起 post 请求
        response = client.get(
            "/reponse_is_list",
            headers=headers,
            # params=params
        )

        # 验证响应状态码
        new_response = response.json()
        assert response.status_code == 200
        assert new_response['message'] == '获取规则执行记录 成功'
    except Exception as e:
        print(e)
#

# post 请求body存在 与 不存在
def test_testcdm3(client):
    """
    测试获取全部数据源类型接口
    """
    json_data = {
        "question": "World"
    }
    try:
        # 发起 post 请求
        response = client.post(
            "/testcdm",
            headers=headers,
            json=json_data
        )

        # 验证响应状态码
        new_response = response.json()
        assert new_response.status_code == 200
    except CustomException as e:
        assert "返回结构不是标准结构" in str(e)
    except Exception as e:
        assert "missing 1 required positional argument" in str(e)
#
def test_testcdm1(client):
    """
    测试获取全部数据源类型接口
    """
    json_data = {
        "question": "World"
    }
    try:
        # 发起 post 请求
        response = client.post(
            "/testcdm",
            headers=headers
        )

        # 验证响应状态码
        new_response = response.json()
        assert response.status_code == 403
    except Exception as e:
        print(e)

# 请求接口中的service 异常，传参异常
#
def test_testservice(client):
    """
    测试获取全部数据源类型接口
    """
    json_data = {
        "question": "World"
    }
    try:
        # 发起 post 请求
        response = client.post(
            "/testservice",
            headers=headers,
            json=json_data
        )

        # 验证响应状态码
        new_response = response.json()
        assert response.status_code == 200
    except CustomException as e:
        assert "返回结构不是标准结构" in str(e)
    except Exception as e:
        assert "返回结构不是标准结构" in str(e)

def test_testservice1(client):
    """
    测试获取全部数据源类型接口
    """
    json_data = {
        "question": "World"
    }
    try:
        # 发起 post 请求
        response = client.post(
            "/testservice1",
            headers=headers,
            json=json_data
        )

        # 验证响应状态码
        new_response = response.json()
        assert response.status_code == 200
    except CustomException as e:
        assert "返回结构不是标准结构" in str(e)
    except Exception as e:
        assert 'missing 1 required positional argument' in str(e)


import asyncio
import inspect
import logging
import os
import time
import functools
import traceback
import json
from starlette.responses import StreamingResponse

from sthg_fastapi_logs.utils.code_utils import handle_method_status, handle_business_code, get_main_traceback
from sthg_fastapi_logs.utils.exception_utils import CustomException
from .enumerate import CodeEnum, HttpStatusCode, RESULT_SUCCESS, BusinessCode
from .log_util import TraceID, local_trace, create_loggers, get_module_line, get_process_funcname, get_main_tracebak, \
    clean_error_message

TOKEN_URL = 'http://192.168.1.243:9103/api/user/userInfoByToken'


def get_process_time(start_time, end_time):
    return '{}'.format(round((end_time - start_time) * 1000, 6))


def get_status(response):
    if hasattr(response, 'status_code'):
        return identify_code(response.status_code)
    return CodeEnum.SUCCESS


def get_trace_id():
    if hasattr(local_trace, 'trace_id'):
        return local_trace.trace_id
    return ''


def get_request_method(request):
    return str(request.url)


def get_ip(request):
    return request.client.host


def get_header(request):
    return dict(request.headers)


async def get_request_params(request):
    params = dict(request.query_params) if request.query_params else "-"
    if not params:
        byte_body = await request.body()
        params = json.loads(byte_body.decode()) if byte_body else "-"
    return params


async def get_response(response):
    # 如果响应是一个 StreamingResponse，我们需要特殊处理
    if isinstance(response, StreamingResponse):
        # 我们需要创建一个新的 StreamingResponse
        # 并且修改它的响应体
        async def new_streaming_response(stream):
            async for chunk in stream:
                # 在这里可以处理每个 chunk，例如记录日志、修改内容等
                yield chunk  # 发送 chunk

        return StreamingResponse(new_streaming_response(response.body_iterator), media_type=response.media_type)
    else:
        # 对于非 StreamingResponse，可以直接修改 response.body
        data = await response.body()
        modified_data = data  # 在这里可以修改数据
        return Response(modified_data, media_type=response.media_type, status_code=response.status_code)
    # return data


def identify_code(code):
    code = int(code)
    ranges = {
        CodeEnum.SUCCESS.value: (199, 207),
        CodeEnum.REEOR.value: (1, 1),
        CodeEnum.PARAM_ERROR.value: (400, 499),
        CodeEnum.INNER_REEOR.value: (500, 10000)
        # 可以添加更多的范围
    }

    for identifier, (start, end) in ranges.items():
        if start <= code <= end:
            return identifier
    # 特殊的 code 处理
    if 0 <= code <= 0:
        return CodeEnum.SUCCESS.value
    return CodeEnum.SUCCESS.value


def get_response_code(response):
    # 拿到 code
    if response and type(response) == dict:
        code = response.get('code') or response.get('Code') or response.get('CODE')
    else:
        code = "-"
    # 转换 code
    if code:
        code = identify_code(code)
    else:
        code = CodeEnum.SUCCESS.value
    return code


def get_response_msg(response):
    if response and type(response) == dict:
        msg = response.get('msg') or response.get('message') or response.get('Message')
    else:
        msg = "-"
    return msg


def get_response_data(response):
    if response and type(response) == dict:
        msg = response
    else:
        msg = "-"
    return msg


def get_request_data(args, kwargs):
    params = {
        'args': args[1:],
        'kwargs': kwargs
    }
    if not params:
        params = '-'
    return params


"""
"time":请求时间
,"traceId":全链路Id
,"method":访问方法
,"status":http状态码
,"code":业务状态码
,"msg": 返回描述，当异常时，可以把简略堆栈放里面
,"resRT": 响应时长
,"logRT": 日志打印耗时
,"guid": 留空
,"requestId":用户访问唯一码
,"userId": 用户id
,"extObj": 扩展信息：包含用Ip,和请求header
,"reqParams": 请求参数
,"reData": 返回参数
"""
from fastapi import (
    FastAPI,
    Request,
    Response
)


# 依赖函数，用于获取所有类型的参数
async def request_params(request):
    # # 路径参数
    path_params = request.path_params
    # 查询参数
    query_params = dict(request.query_params)

    content_type = request.headers.get('content-type') if request.headers.get('content-type') else ''
    # 表单数据
    form_data = {}
    #不可识别数据
    others = ''
    # 请求体数据
    body_data = {}
    try:
        if content_type and 'application/json' in content_type:
            body_data = await request.body()
            body_data = json.dumps(body_data.decode())
        elif content_type and 'multipart/form-data' in content_type:
            if 'multipart/form-data; boundary=' in content_type:
                form = await asyncio.wait_for(request.form(), timeout=10)
                has_file = False
                filename = '-'
                for field_name, value in form.items():
                    if hasattr(field_name, 'filename'):
                        has_file = True
                        break
                if has_file:
                    form_data["file_upload"] = 1
                else:
                   form = await request.form()
                   form_data = {key: value for key, value in form.items()}
            else:
                form = await request.form()
                form_data = {key: value for key, value in form.items()}
        elif content_type and 'application/x-www-form-urlencoded' in content_type:
            form = await request.form()
            form_data = {key: value for key, value in form.items()}
        else:
            others = '-'
    except Exception as e:
        pass
    params_input = {}
    if query_params:
        params_input['params'] = query_params
    if form_data:
        params_input['form_data'] = form_data
    if body_data:
        params_input['body'] = body_data
    if others:
        params_input['others'] = others
    return params_input


# 声明全局变量
default_config = {
    "access": {
        "is_console": False,
        "is_msg": False,
        "is_request": False,
        "is_response": False,
        "logger_level": logging.DEBUG,
        "console_level": logging.INFO,
        "acc_when": "W6",
        "backupCount": 0
    },
    "error": {
        "is_console": False,
        "is_msg": False,
        "is_request": False,
        "is_response": False,
        "logger_level": logging.ERROR,
        "console_level": logging.ERROR,
        "acc_when": "W6",
        "backupCount": 0
    },
    "all_info": {
        "is_console": False,
        "is_msg": False,
        "is_request": False,
        "is_response": False,
        "logger_level": logging.INFO,
        "console_level": logging.INFO,
        "acc_when": "W6",
        "backupCount": 0
    },
    "server": {
        "is_console": False,
        "is_msg": False,
        "is_request": False,
        "is_response": False,
        "logger_level": logging.DEBUG,
        "console_level": logging.INFO,
        "acc_when": "W6",
        "backupCount": 0
    }
}


def init_log_config(config: dict[str, dict] = None):
    global default_config,acc_logger, server_logger, error_logger, all_info
    if config:
        for k, v in config.items():
            default_config[k] = v
    acc_logger, server_logger, error_logger, all_info = create_loggers(default_config)


def get_module_func_by_router(request):
    route = request.scope.get("route")
    if route:
        endpoint = route.endpoint
        # 获取函数的定义信息
        source_lines, start_line = inspect.getsourcelines(endpoint)
        method_name = endpoint.__name__
        module_file = inspect.getmodule(endpoint).__file__
        return module_file, method_name, start_line
    else:
        raise Exception('route not found')


def register_log_middleware(app: FastAPI):
    # 全局变量,以便通过中间件引入,可以进行配置


    @app.middleware("http")
    async def log_middleware(request: Request, call_next):
        start_time = time.time()
        _request_id_key = "X-Request-Id"
        _trace_id_key = "X-Request-Id"
        _trace_id_val = request.headers.get(_trace_id_key)
        if _trace_id_val:
            TraceID.set_trace(_trace_id_val)
            TraceID.set_trace(_trace_id_val)
        else:
            TraceID.new_trace()
            _trace_id_val = TraceID.get_trace()
        local_trace.trace_id = TraceID.get_trace()
        local_trace.request = request
        moduler_func_line = get_process_funcname(app, request)

        # content_type = request.headers.get('content-type', '')


        async  def set_body(request: Request):
            receive_ = await  request._receive()
            async def receive():
                return receive_
            request._receive = receive

        await set_body(request)

        param_input = await request_params(request)
        if not param_input:
            param_input = "-"

        try:
            response = await call_next(request)

            process_time = get_process_time(start_time, time.time())
            log_time_start = time.time()

            msg = get_response_msg(response)
            status_code = handle_method_status(response.status_code)
            code = handle_business_code(response.status_code)
            body = b""
            if hasattr(response, "body_iterator"):
                async for chunk in response.body_iterator:
                    body += chunk
            try:
                # 将字节类型的 body 解码为字符串，再解析为 JSON 对象
                json_data = json.loads(body.decode('utf-8'))

            except json.JSONDecodeError:
                # 处理 JSON 解析错误
                json_data = '-'

            if status_code != "SUCCESS":
                if type(json_data) is dict:
                    msg = get_response_msg(json_data)
                    module_file, method_name, start_line = get_module_func_by_router(request)
                    msg = f'Traceback (most recent call last):\nFile\t"{module_file}",\tline{start_line},\tin\t{method_name}\traise\tException({msg})'
                json_data = '-'

            info_kwargs = {
                'request': request,
                'process_time': process_time,
                "status_code": status_code,
                "code": code,
                'return_desc': HttpStatusCode.get_description(response.status_code),
                'msg': msg,
                'moduler_func_line': moduler_func_line,
                'param_input': param_input,
                'respData': json_data if json_data else '_',
                "log_time_start": log_time_start
            }
            await get_all_info_log_str(**info_kwargs)
            # 重新创建响应对象
            new_response = Response(content=body, status_code=response.status_code, headers=dict(response.headers))
            new_response.headers["X-Request-Id"] = str(_trace_id_val)
            new_response.headers["X-Process-Time"] = str(process_time)
            return new_response
        except AttributeError as e:
            process_time = get_process_time(start_time, time.time())
            log_time_start = time.time()
            # 获取当前请求对应的路由处理函数
            module_file, method_name, start_line = get_module_func_by_router(request)
            custom_stack = f'Traceback (most recent call last):\nFile\t"{module_file}",\tline{start_line},\tin\t{method_name}\traise\tAttributeError({e})'

            status_code = handle_method_status(500)
            code = handle_business_code(500)
            info_kwargs = {
                'request': request,
                'process_time': process_time,
                "status_code": status_code,
                "code": code,
                'return_desc': HttpStatusCode.get_description(500),
                'msg': custom_stack,
                'moduler_func_line': moduler_func_line,
                'param_input': param_input,
                'respData': '-',
                "log_time_start": log_time_start
            }
            await get_all_info_log_str(**info_kwargs)
            raise e

        except Exception as e:
            process_time = get_process_time(start_time, time.time())
            log_time_start = time.time()
            status_code = handle_method_status(500)
            code = handle_business_code(500)
            msg = get_main_traceback(traceback.format_exc())
            module_file, method_name, start_line = get_module_func_by_router(request)
            custom_stack = f'Traceback (most recent call last):\nFile\t"{module_file}",\tline{start_line},\tin\t{method_name}\traise\tException({e})'
            info_kwargs = {
                'request': request,
                'process_time': process_time,
                "status_code": status_code,
                "code": code,
                'return_desc': HttpStatusCode.get_description(500),
                'msg': custom_stack,
                'moduler_func_line': moduler_func_line,
                'param_input': param_input,
                'respData': '-',
                "log_time_start": log_time_start
            }
            await get_all_info_log_str(**info_kwargs)
            raise e


# 记录api执行日志
def access_log(**access_kwargs):
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            start_time = time.time()
            # 尝试从参数中获取 Request 对象
            request = None
            for arg in args:
                if isinstance(arg, Request):
                    request = arg
                    break
            if not request:
                request = local_trace.request

            params = await request_params(request)
            moduler_func_line = ''
            return_desc = '-'
            try:
                # 获取code
                msg = "-"
                request = local_trace.request
                method_name = func.__name__
                module = inspect.getmodule(func)
                if module is not None and hasattr(module, '__file__'):
                    # 获取模块对应的文件路径
                    project_root = os.getcwd()
                    # current_dir = os.path.dirname(os.path.abspath(project_root))
                    module_file = module.__file__
                    relative_path = os.path.relpath(module_file, project_root)
                    module_path = relative_path.replace(os.sep, '.')[:-3]  # 去除 .py 扩展名
                    new_module_path = module_path.split('.')[-1]
                    moduler_func_line = "{}.{}".format(new_module_path, method_name)

                if asyncio.iscoroutinefunction(func) or 'function' not in str(type(func)):
                    response = await func(*args, **kwargs)
                    if response and not isinstance(response, dict):
                        response = await response
                else:
                    response = func(*args, **kwargs)
                process_time = get_process_time(start_time, time.time())
                log_time_start = time.time()

                if isinstance(response, dict):
                    if 'code' in response or 'http_code' in response:
                        if 'code' in response:
                            code = response['code']
                        else:
                            code = response['http_code']
                    else:
                        raise CustomException("返回结构不是标准结构")
                else:
                    raise CustomException("返回结构不是标准结构")

                status_code = handle_method_status(code)
                busiCode = handle_business_code(code)
                if code in [0, 1]:
                    if code == 0:
                        return_desc = f"请求成功"
                    if code == 1:
                        return_desc = f"请求失败"
                else:
                    return_desc = HttpStatusCode.get_description(code)

                access_info = {
                    'request': request,
                    'process_time': process_time,
                    "status_code": status_code,
                    "code": busiCode,
                    'return_desc': return_desc,
                    'msg': msg,
                    'moduler_func_line': moduler_func_line,
                    'param_input': params,
                    'respData': response if response else '-',
                    "log_time_start": log_time_start
                }
                await get_access_log_str(**access_info)
            except CustomException as e:
                process_time = get_process_time(start_time, time.time())
                log_time_start = time.time()
                file_path = inspect.getfile(func)
                # 获取文件所在的目录路径
                current_dir = os.path.dirname(os.path.abspath(file_path))
                moduler_func_line = moduler_func_line if moduler_func_line else get_module_line(traceback.format_exc(),
                                                                                                project_path=current_dir)
                _, start_line = inspect.getsourcelines(func)
                custom_stack = f'Traceback (most recent call last):\nFile\t"{module_file}",\tline{start_line},\t in\t{method_name}\traise CustomException("{e}")'
                request = local_trace.request
                access_info = {
                    'request': request,
                    'process_time': process_time,
                    "status_code": 'SUCCESS',
                    "code": 'InnerError',
                    'return_desc': HttpStatusCode.get_description(500),
                    'msg': custom_stack,
                    'moduler_func_line': moduler_func_line,
                    'param_input': params,
                    'respData': '-',
                    "log_time_start": log_time_start
                }
                await get_access_log_str(**access_info)
                raise CustomException(custom_stack)
            except ValueError as e:
                process_time = get_process_time(start_time, time.time())
                log_time_start = time.time()
                file_path = inspect.getfile(func)
                # 获取文件所在的目录路径
                current_dir = os.path.dirname(os.path.abspath(file_path))
                moduler_func_line = moduler_func_line if moduler_func_line else get_module_line(traceback.format_exc(),
                                                                                                project_path=current_dir)
                msg = get_main_traceback(traceback.format_exc())
                request = local_trace.request
                response = Response(status_code=500)
                access_info = {
                    'request': request,
                    'process_time': process_time,
                    "status_code": handle_method_status(response.status_code),
                    "code": handle_business_code(response.status_code),
                    'return_desc': HttpStatusCode.get_description(response.status_code),
                    'msg': msg,
                    'moduler_func_line': moduler_func_line,
                    'param_input': params,
                    'respData': '-',
                    "log_time_start": log_time_start
                }
                await get_access_log_str(**access_info)
                raise ValueError(e)
            except Exception as exc:
                process_time = get_process_time(start_time, time.time())
                log_time_start = time.time()
                file_path = inspect.getfile(func)
                # 获取文件所在的目录路径
                current_dir = os.path.dirname(os.path.abspath(file_path))

                moduler_func_line = moduler_func_line if moduler_func_line else get_module_line(traceback.format_exc(),
                                                                                                project_path=current_dir)
                exc_type = type(exc)
                exc_value = exc
                exc_tb = exc.__traceback__
                # 调用 format_exception 函数格式化异常信息
                formatted_exception = traceback.format_exception(exc_type, exc_value, exc_tb)
                # 打印格式化后的异常信息
                main_error = "Traceback (most recent call last):\n" + get_main_tracebak(''.join(formatted_exception),
                                                                                        project_path=current_dir)
                #
                request = local_trace.request
                response = Response(status_code=500)
                access_info = {
                    'request': request,
                    'process_time': process_time,
                    "status_code": 'ERROR',
                    "code": 'InnerError',
                    'return_desc': HttpStatusCode.get_description(response.status_code),
                    'msg': main_error,
                    'moduler_func_line': moduler_func_line,
                    'param_input': params,
                    'respData': '-',
                    "log_time_start": log_time_start
                }
                await get_access_log_str(**access_info)
                raise exc
            return response

        return wrapper

    return decorator


"""
"time":请求时间
,"traceId":全链路Id
,"method":访问方法
,"status":http状态码
,"code":业务状态码
,"msg": 返回描述，当异常时，可以把简略堆栈放里面
,"resRT": 响应时长
,"logRT": 日志打印耗时
,"reqParams": 请求参数
,"reData": 返回参数
"""


# 记录方法执行日志
def service_log():
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            msg = '-'
            params = get_request_data(args, kwargs)
            method_name = func.__name__
            module = inspect.getmodule(func)
            if module is not None and hasattr(module, '__file__'):
                # 获取模块对应的文件路径
                project_root = os.getcwd()
                module_file = module.__file__
                relative_path = os.path.relpath(module_file, project_root)
                module_path = relative_path.replace(os.sep, '.')[:-3]  # 去除 .py 扩展名
                new_module_path = module_path.split('.')[-1]
                moduler_func_line = "{}.{}".format(new_module_path, method_name)
            start_time = time.time()

            if asyncio.iscoroutinefunction(func) or 'function' not in str(type(func)):
                response = await func(*args, **kwargs)
                if not isinstance(response, dict):
                    response = await response
            else:
                response = func(*args, **kwargs)
            process_time = get_process_time(start_time, time.time())
            log_time_start = time.time()
            reData = get_response_data(response)
            if not params: params = '-'
            await get_service_log_str(moduler_func_line, process_time, log_time_start, RESULT_SUCCESS, "OK",
                                      "业务处理成功",
                                      msg, params, reData)
            return response

        return wrapper

    return decorator


#
def class_decorator(func):
    @functools.wraps(func)
    async def wrapper(*args, **kwargs):
        msg = "-"
        reData = '-'
        params = get_request_data(args, kwargs)

        method_name = func.__name__
        module = inspect.getmodule(func)
        if module is not None and hasattr(module, '__file__'):
            # 获取模块对应的文件路径
            project_root = os.getcwd()
            current_dir = os.path.dirname(os.path.abspath(project_root))
            module_file = module.__file__
            relative_path = os.path.relpath(module_file, current_dir)
            module_path = relative_path.replace(os.sep, '.')[:-3]  # 去除 .py 扩展名
            new_module_path = module_path.split('.')[-1]
            moduler_func_line = "{}.{}".format(new_module_path, method_name)
        start_time = time.time()
        response = func(*args, **kwargs)
        process_time = get_process_time(start_time, time.time())
        log_time_start = time.time()
        reData = response if response else reData
        await get_service_log_str(moduler_func_line, process_time, log_time_start, "SUCCESS", 'OK', '业务处理成功', msg,
                                  params, reData)
        return response

    return wrapper


def class_log(cls):
    for name, method in vars(cls).items():
        if callable(method) and name != '__init__':  # 排除__init__方法
            setattr(cls, name, class_decorator(method))
    return cls


async def get_access_log_str(**access_kwargs):
    # acc_logger, server_logger, error_logger, all_info = create_loggers(default_config)
    try:
        request = access_kwargs['request']
        request_header = get_header(request)
        guid = request_header.get('X-GUID') if request_header.get('X-GUID') else "-"
        requestId = TraceID.get_trace() if TraceID.get_trace() else "-"
        if request_header.get('X-User-ID'):
            userId = request_header.get('X-User-ID')
        elif request_header.get('user_id'):
            userId = request_header.get('user_id')
        else:
            userId = "-"
        header = {"user_ip": get_ip(request), "host": request_header['host'], 'user_id': userId}
        msg = clean_error_message(access_kwargs['msg'])
        moduler_func_line = str(access_kwargs['moduler_func_line']).replace('...', '')
        status_code = access_kwargs['status_code']
        code = access_kwargs['code']
        return_desc = access_kwargs['return_desc']
        process_time = access_kwargs['process_time']
        param_input = access_kwargs['param_input']
        respData = access_kwargs['respData']
        log_time_start = access_kwargs['log_time_start']
        logRT = get_process_time(log_time_start, time.time())
        info_msg = f"""{moduler_func_line}\t {status_code}\t {code}\t {return_desc}\t {process_time}\t {logRT}\t {guid}\t {userId}\t {requestId}\t header-{header}\t {param_input}\t {respData}\t {msg}"""

        acc_logger.info(str(info_msg))
    except Exception as e:
        error_logger.error('access log error :{}'.format(str(e)))


async def get_all_info_log_str(**info_kwargs):
    # acc_logger, server_logger, error_logger, all_info = create_loggers(default_config)
    try:
        request = info_kwargs['request']
        request_header = get_header(request)
        guid = request_header.get('X-GUID') if request_header.get('X-GUID') else "-"
        requestId = TraceID.get_trace() if TraceID.get_trace() else "-"
        if request_header.get('X-User-ID'):
            userId = request_header.get('X-User-ID')
        elif request_header.get('user_id'):
            userId = request_header.get('user_id')
        else:
            userId = "-"
        header = {"user_ip": get_ip(request), "host": request_header['host'], 'user_id': userId}
        moduler_func_line = str(info_kwargs['moduler_func_line']).replace('...', '')
        status_code = info_kwargs['status_code']
        code = info_kwargs['code']
        process_time = info_kwargs['process_time']
        msg = clean_error_message(info_kwargs['msg'])
        param_input = info_kwargs['param_input']
        respData = info_kwargs['respData']
        return_desc = info_kwargs['return_desc']
        log_time_start = info_kwargs['log_time_start']
        logRT = get_process_time(log_time_start, time.time())
        info_msg = f"""{moduler_func_line}\t {status_code}\t {code}\t {return_desc}\t {process_time}\t {logRT}\t {guid}\t {userId}\t {requestId}\t header-{header}\t {param_input}\t {respData}\t {msg}\t """
        all_info.info(str(info_msg))
    except Exception as e:
        error_logger.error('access log error :{}'.format(str(e)))


async def get_error_log(request, status_code, code, msg, module_line="-"):
    # acc_logger, server_logger, error_logger, all_info = create_loggers(default_config)
    try:
        t1 = time.time()
        request_header = get_header(request)
        guid = request_header.get('X-GUID') if request_header.get('X-GUID') else "-"
        requestId = TraceID.get_trace() if TraceID.get_trace() else "-"
        if request_header.get('X-User-ID'):
            userId = request_header.get('X-User-ID')
        elif request_header.get('user_id'):
            userId = request_header.get('user_id')
        else:
            userId = "-"
        header = {"user_ip": get_ip(request), "host": request_header['host'], 'user_id': userId}
        module_line = str(module_line).replace('...', '')
        logRT = get_process_time(t1, time.time())
        info_msg = f'{module_line}\t {status_code}\t {code}\t {logRT}\t {guid}\t {userId}\t {requestId}\t header-{header}\n{msg}'
        error_logger.error(str(info_msg))
    except Exception as e:
        error_logger.error('access log error :{}'.format(str(e)))
        raise e


async def get_service_log_str(method, process_time, log_time_start, status_code, code, return_desc, msg, reqParam,
                              reData):
    # acc_logger, server_logger, error_logger, all_info = create_loggers(default_config)
    try:
        msg = clean_error_message(msg)
        logRT = get_process_time(log_time_start, time.time())
        service_msg = f"""{method}\t {status_code}\t {code}\t {return_desc}\t {process_time}\t {logRT}\t {reqParam}\t {reData} {msg}"""
        server_logger.debug(str(service_msg))
    except Exception as e:
        error_logger.error('service log error :{}'.format(str(e)))



"""
@Author  ：duomei
@File    ：code_utils.py
@Time    ：2025/3/11 13:48
"""
import inspect
import os
import re
import traceback
from http.client import HTTPException

from sthg_fastapi_logs.log_util import get_main_tracebak


def exception_module_func(request,exc,project_folder):
    moduler_func_line = '_'
    route = request.scope.get("route")
    project_root = ""
    main_error = "-"
    method_name = "_"
    if route:
        endpoint = route.endpoint
        # 获取函数的定义信息
        method_name = endpoint.__name__
        module_file = inspect.getmodule(endpoint).__file__
        _,line = inspect.getsourcelines(endpoint)

        # 获取模块对应的文件路径
        project_root = os.path.dirname(os.path.abspath(module_file))
        relative_path = os.path.relpath(module_file, project_root)
        module_path = relative_path.replace(os.sep, '.')[:-3]  # 去除 .py 扩展名
        moduler_func_line = "{}.{}".format(module_path.replace('...',''), method_name)
    else:
        raise HTTPException(404, "路由未找到")
    if project_root:
            exc_type = type(exc)
            exc_value = exc
            exc_tb = exc.__traceback__
            # 调用 format_exception 函数格式化异常信息
            formatted_exception = traceback.format_exception(exc_type, exc_value, exc_tb)
            # project_path = os.path.dirname(os.path.abspath(project_root))
            # 打印格式化后的异常信息
            main_error = get_main_tracebak(''.join(formatted_exception), project_path=project_root)

    return moduler_func_line,main_error,method_name,project_root,line



# 假设的判断是否为标准结构体的函数
def is_standard_structure(response):
    """
    判断结果是否为标准结构体
    :param response: 方法返回的结果
    :return: True 表示是标准结构体，False 表示不是
    """
    # 这里简单假设标准结构体是字典且包含 'status' 和 'business_code' 字段
    if isinstance(response, dict):
        # 假设标准结构体必须包含 'code' 和'message' 键
        if 'code' in response or 'http_code' in response:
            return True
    return False


# 假设的获取 HTTP 状态码的函数
def get_http_status_code(response):
    """
    从结果中获取 HTTP 状态码
    :param response: 方法返回的结果
    :return: HTTP 状态码，如果不存在则返回 None
    """
    return response.get('http_status_code') if isinstance(response, dict) else None


def get_main_traceback(stack_trace):
    # 定义正则表达式模式，匹配最后一个 raise 语句及相关文件行信息
    pattern = r'File "(.*?)", line (\d+), in (.*?)\s+raise (.*)'
    matches = re.findall(pattern, stack_trace)

    if matches:
        # 取最后一个匹配结果
        file_path, line_number, function_name, raise_statement = matches[-1]
        main_error = f'Traceback (most recent call last):\n File\t"{file_path}",\tline{line_number},\tin\t{function_name}\traise\t{raise_statement}'
    else:
        main_error = "-"
    return main_error


class StandardResult:
    def __init__(self, code, data, message):
        self.code = code
        self.data = data
        self.message = message


# 处理方法结果状态
def handle_method_status(code):
    if isinstance(code, int):
        if 200 <= code < 300:
            return "SUCCESS"
        elif 300 <= code < 400:
            return "REDIRECTION"
        elif code == 400:
            return "BAD_REQUEST"
        elif code == 401:
            return "UNAUTHORIZED"
        elif code == 403:
            return "FORBIDDEN"
        elif code == 404:
            return "NOT_FOUND"
        elif code == 405:
            return "ERROR"
        elif code >= 500:
            return "ERROR"
        elif code == 0:
            return "SUCCESS"
        elif code == 1:
            return "ERROR"
    elif isinstance(code, str):
        return code
    else:
        return "ERROR"


# 处理业务编码
def handle_business_code(code):
    if isinstance(code, int):
        if 200 <= code < 300:
            return "OK"
        elif 300 <= code < 400:
            return "Redirection"
        elif code == 400:
            return "BadRequest"
        elif code == 401:
            return "Unauthorized"
        elif code == 403:
            return "Forbidden"
        elif code == 404:
            return "NotFound"
        elif code == 405:
            return "InnerError"
        elif code >= 500:
            return "InnerError"
        elif code == 0:
            return "OK"
        elif code == 1:
            return "InnerError"
    elif isinstance(code, str):
        return code
    else:
        return "ERROR"



__version__ = "0.1.17"

from .exception import register_log_exception as register_log_exception
from .log_wrapper import register_log_middleware as register_log_middleware
# from .log_wrapper import class_log as class_log
# from .log_wrapper import access_log as access_log
# from .log_wrapper import service_log as service_log
